#include "m_pd.h"
#define fs 44100
#define M 44100

static t_class *delayreverb_class;

typedef struct _delayreverb
{
  t_object x_obj;
  float x_f;
  float ring_buffer[M];
  float d;
  float a;
  int index;
} t_delayreverb;

static t_int *delayreverb_perform(t_int *w)
{
  t_delayreverb *x = (t_delayreverb*)(w[1]);
  t_float *s0 = (t_float*)(w[2]);
  t_float *s1 = (t_float*)(w[3]);
  int N = (int)(w[4]);
  
  int n;
  int d = (int)(x->d * 0.001 * fs);
  float dry, wet;
  
  for (n = 0; n < N; n++)
  {
    dry = s0[n];
    wet = x->ring_buffer[x->index] * x->a;
    s1[n] = dry + wet;
    
    x->ring_buffer[x->index] = s1[n];
    x->index++;
    if (x->index >= d)
    {
      x->index = 0;
    }
  }
  
  return (w + 5);
}

void delayreverb_dsp(t_delayreverb *x, t_signal **sp)
{
  dsp_add(delayreverb_perform, 4, (t_int*)x, sp[0]->s_vec, sp[1]->s_vec, sp[0]->s_n);
}

void *delayreverb_new(t_floatarg a0, t_floatarg a1)
{
  t_delayreverb *x = (t_delayreverb *)pd_new(delayreverb_class);
  
  if (a0 < 0) a0 = 0;
  if (a0 > 1000) a0 = 1000;
  if (a1 < 0) a1 = 0;
  if (a1 > 1) a1 = 1;
  
  x->d = a0;
  x->a = a1;
  x->index = 0;
  
  outlet_new(&x->x_obj, gensym("signal"));
  floatinlet_new(&x->x_obj, &x->d);
  floatinlet_new(&x->x_obj, &x->a);
  x->x_f = 0;
  
  return (void *)x;
}

void delayreverb_tilde_setup(void)
{
  delayreverb_class = class_new(gensym("delayreverb~"),
    (t_newmethod)delayreverb_new, 0, sizeof(t_delayreverb), 0, A_DEFFLOAT, A_DEFFLOAT, 0);
  CLASS_MAINSIGNALIN(delayreverb_class, t_delayreverb, x_f);
  class_addmethod(delayreverb_class, (t_method)delayreverb_dsp, gensym("dsp"), 0);
}
